<?php

/**
 * @file
 * Plugin for entity behaviors skrollr.
 */
$plugin = array(
  'name' => 'skrollr',
  'label' => t('Add skrollr behaviors to entities.'),
  'handler' => 'entity_behaviors_skrollr_behavior',
);

function entity_behaviors_skrollr_behavior($selector, $entity) {
  if (($library = libraries_load(EBS_SKROLLR_LIBRARY)) && !empty($library['loaded'])) {
    $options = array();
    $wrapper = entity_metadata_wrapper('field_collection_item', $entity);
    foreach ($wrapper->{EBS_SKROLLR_FIELD}->getIterator() as $delta => $value) {
      // Get data attributes.
      $data = $value->value();
      // Validate data.
      if (!entity_behaviors_skrollr_validate_data($data)) {
        continue;
      }
      // Sanitize data.
      list($attr_name, $attr_value) = entity_behaviors_skrollr_get_sanitized_data($data);
      // Fill in data attributes.
      $options[$attr_name] = $attr_value;
    }
    // If options is not empty then load library.
    if (!empty($options)) {
      $js_settings = array($selector => $options);
      drupal_add_js(array('entity_behaviors_skrollr' => $js_settings), 'setting');
      // Add skrollr.js plugin.
      drupal_add_js(drupal_get_path('module', 'entity_behaviors_skrollr') . '/plugins/entity_behaviors/skrollr.js');
    }
  }
}

/**
 * Validate skrollr data attributes.
 *
 * @param string $data
 *   A skrollr configuration line. For example:
 *   data-0="background-color:rgb(0,0,255);transform[bounce]:rotate(0deg);"
 * @return boolean
 *   TRUE if validation is successful, FALSE otherwise.
 */
function entity_behaviors_skrollr_validate_data($data) {
  // Split by "=" sign and validate its existence.
  $pieces = explode("=", $data, 2);
  if (count($pieces) != 2) {
    return FALSE;
  }
  // Fill in the variables.
  list($attr_name, $attr_value) = $pieces;
  // Regex for skrollr data-attribute name:
  // data-[_constant]-[offset]-[anchor]-[anchor].
  // The regular expression is directly copied from skrollr.js.
  $data_attribute_name_regex = '/^data(?:-(_\w+))?(?:-?(-?\d*\.?\d+p?))?(?:-?(start|end|top|center|bottom))?(?:-?(top|center|bottom))?$/';
  // Validate the skrollr data-attribute.
  if (!preg_match($data_attribute_name_regex, $attr_name)) {
    return FALSE;
  }

  return TRUE;
}

/**
 * Get sanitized skrollr data attributes.
 *
 * @param string $data
 *   A skrollr configuration line. For example:
 *   data-0="background-color:rgb(0,0,255);transform[bounce]:rotate(0deg);"
 * @return array
 *   An array containing attribute name and attribute value.
 */
function entity_behaviors_skrollr_get_sanitized_data($data) {
  // Fill in the variables.
  list($attr_name, $attr_value) = explode("=", $data, 2);

  // Sanitize the attribute key.
  $attr_name = check_plain($attr_name);
  // Sanitize the attribute value and remove first and last double quotes.
  $attr_value = check_plain(trim($attr_value, '"'));

  return array($attr_name, $attr_value);
}
